@@ -248,7 +248,7 @@ fbOverlayCopyWindow(WindowPtr	pWin,
 	{
 	    REGION_TRANSLATE(pScreen, &layerRgn[i], -dx, -dy);
 	    pPixmap = pScrPriv->layer[i].u.run.pixmap;
-	    fbCopyRegion (&pPixmap->drawable, &pPixmap->drawable,
+	    miCopyRegion (&pPixmap->drawable, &pPixmap->drawable,
 			  0,
 			  &layerRgn[i], dx, dy, pScrPriv->CopyWindow, 0,
 			  (void *)(long) i);
